# nanosoc aes128 example Accelerator System Project

This repo is the top-level repository which contains all the relavant IP for integrating a secworks AES128 hardware accelerator with the SoC Labs provided nanosoc chip design IP clnewd from `https://git.soton.ac.uk/soclabs/accelerator-project` and adapted as a reference design for 128-bit key AES cipher acceleration.

The IP core used in this example is based on the secworks aes project core: https://github.com/secworks/aes secworks-aes

This design instantiates a custom (AMBA-AHB) wrapper around the AES core to implement a memory-mapped 128-bit AES encrypt/decrypt accelerator that can be used as a software-driven peripheral or a semi-autonomous DMA subystem when 128-bit keys and variable length data payloads can be set up as scatter/gather descriptor chains for background processing.


### Cloning this repository
---

This Repository contains multiple sub-repositories. In order to clone them with this top-level system repository, please use the following command:

`git clone --recurse https://git.soton.ac.uk/soclabs/aes-128-project.git`

then set current directory to the top of this project:

`cd aes-128-project`

Add the AES IP core (packaged as a fusesoc component):

`git submodule add https://github.com/secworks/aes.git secworks-aes`

This specific `secworks-aes` directory repository name is included in `.gitmodules`  configuration; this ensures the set_env.sh script will pull in IP core repository when updates are made.

The /env/dependency_env.sh file is already modified to include the accelerator directory:

`export ACCELERATOR_DIR="$SOCLABS_PROJECT_DIR/secworks-aes"`


### Setting up the Project Environment
---

Every time you wish to run commands in this project, you will need to make sure the set environment script has been run for your current terminal session. This is done by moving to the top-level of the project and running:

`source set_env.sh` 

This sets the environment variables related to this project and creates visability to the scripts in the flow directory. 

### Updating Subrepositories
---

Once you have run a `source set_env.sh` in your current terminal, you are then able to update all your repositories to their latest version by running:

`socpull`

This runs a git pull on all repositories in your project.

### Project Structure
---

The core of the SoC is NanoSoC. This is an example, configurable system that is the main framework. It has many different memory-mapped regions, one of which is designed for the connection of accelerator subsystems called the expansion region.

The expansion region is able to instantiate an accelerator_subsystem by default. This means that anyone using NanoSoC as a platform for accelerator experimentation will need to build an `accelerator_subsystem` rtl module.

This AES128 project example file is instantiatd in `/system/src/accelerator_subsystem.v`. This provides the connections at this level for an AHB subordinate or target control register file, Direct Memory Address Controller input and out data (128-bit as 4x32-bit words) request signals to the DMA230 controller used in this design, and interrupt request signals to the NanoSoC CPU (Arm Cortex-M0 microprocessor).

The DMA230 controller is configured in the project nanosoc_config file:

`DMA_0_PL230_INCLUDE := yes`

### Using the makefiles and FLISTS
---

Simulation, FPGA implementation, and ASIC synthesis can all be performed from the main `makefile` in the nanosoc_tech repository.
This project includes the `/flist/project/accelerator`. This uses the recommended environment variables, the top level of your project will be `$SOCLABS_PROJECT_DIR`.
In this case, the `soclabs_ahb_aes128_ctrl.v` file is explict include directives to add the secworks-aes src verilog RTL files.


## htmlgen design visualisation
--

A tool is provided to generate an html documentation tree to traverse and explore the design hierarchy:

`htmlgen -f $SOCLABS_PROJECT_DIR/flist/project/top.flist`
or
 `htmlgen`
populates the html/top/build/ directiory. Open `nanosoc_tb.html` to explore from the testbench down in to the design.


## Accelerator Subsystem
--

This example `accelerator_subsystem` directly contains the aes128 accelerator RTL in the `system/src/` directory. with the wrapper named `system/src/accelerator_subsystem.v` that nanosoc instantiates as the generic wrapper interface.

### Accelerator Wrapper
---

The example AES128 accelerator has a native AMBA 32-bit AHB interface so the wrapper components within the `accelerator_wrapper_tech` repository are not used in this case.


## Running the simulation
--

The simulation scripts are contained in `$SOCLABS_PROJECT_DIR/simulate/socsim/` directory.

This project runs all the standard nanosoc validation tests provided in the `nanosoc_tech/testcodes` directory to prove the nanosoc microcontrolleris functionally operational.

The currently supported simulators are VCS, Excelium and QuestaSim. The default in the scriptsis QuestaSim (run_mti). (Edit the simulator target in nanosoc_tech/system/makefile for the simulator EDA tool used).

You can also run `sim_%SIMULATOR%` and this will run the simulation from the GUI.

To run the simulation the 'socsim' command executes the makefile in the 'nanosoc_tech' microcontroller framework.

To ensure simulation tools and environment are setup correctly a simple testcase should run and produce a `TEST PASSED` message satisfactorily.

`socsim test_nanosoc TESTNAME=hello ACCELERATOR=yes`

in fact the simulation and regression scripts all include the `ACCELERATOR=yes` configuration:

`socsim test_nanosoc TESTNAME=hello`


### Acccelerator specific validation programs
---

All the standard validation tests for nanosoc should run and pass as normal on the Arm Cortex-M0 processor (as provided in nanosoc_tech/testcodes directory).
Additional specifc AES128 accelerator validation programs are provided in the `system/testcodes` directory to be run from the Arm Cortex-M0 processor;
these have been added to the recommended `/system/testcodes/$TESTNAME` directory. 

This AES128 project design instantiates a custom (AMBA-AHB) wrapper around the AES core to implement a memory-mapped 128-bit AES encrypt/decrypt accelerator that can be used as a software-driven peripheral or a semi-autonomous DMA subystem when 128-bit keys and variable length data payloads can be set up as scatter/gather descriptor chains for background processing.

The software only version of the tests runs on the Arm Cortex-M0 CPU only and uses `memcpy` operations to write 128-bit keys and 128-bit plaintext input data and 128-bit reads to extract the cipher blocks.

Usage is:

`socsim test_nanosoc TESTNAME=aes128_tests_memcpy`

The DMA validation tests perform the same functional verification but use the Arm DMA230 DMA controller programming model to test 32-bit, 16-bit and 8-bit bus transactions to transfer keys and data blocks. This validates the memory-mapped IO operations correctly auot-trigger the AES on the final byte transfer of key or data text blocks - which ensures that non-word-aligned data can be processed by the accelerator. 

This runs the integration test program on the Arm Cortex-M0 processor using the 'test_aes128.sh' script provided in the simulate/socsim directory and the logs are produced in the simulate/sim/test_aes128/logs directory.

The next test validates the DMA Controller IP is correctly installed and instantiated. In cut-down systems (such as the QuickStart nanosoc alternative) there is no DMA controller and this test will fail while the `memcpy` software will run. 

`socsim test_nanosoc TESTNAME=aes128_tests_dma230`


The full set of accelerator regression tests plus the nanosoc system tests can then be run with:

`socsim regression_accelerator`

the logfiles are written to 
`$SOCLABS_PROJECT_DIR/simulate/sim/regression_<date_stamp>/`


NOTE: To enable the makefiles to find the addional testcode programs, the extra aes128 test names have been added to the `/system/testcodes/software_list.txt` file, as recommended.


## FPGA Builds
==

The FPGA implementation instanties a full `nanosoc_chip` level of hierarchy - the pad-ring final chip level is replaced by an FPGA framework that supports the provision of clock, reset, serial wire debug and host communication channels, to support validation of the chip design much like the simulation top-level testbench.
 
We currently have build files for the ARM MPS3 "Microcontroller Prototyping Platform", and four main-stream Xilinx ZYNQ-pbased that have full support for running the PYNQ Python & Jupyter notebook as network appliances: ZCU104, PYNQ-Z2, KRIA KR260 and KRIA KV260.

The acceptable targets are defined in the `/nanosoc_tech/fpga/makefile.targets` and are `mps3, zcu104, z2, kr260, kv260` currently.

To build the bitfiles you can run the `make build_fpga FPGA=%target% ACCELERATOR=yes` from the `$SOCLABS_PROJECT_DIR/nanosoc_tech/` directory.
(The `ACCELERATOR=yes` does have to be explicit for all make targets run in the `nanosoc_tech` directory)

This script will run the vivado build scripts (updated to AMD/Xilinx Vivado 2024.1). The output files required to configure the FPGA (Overlay in the case of the PYNQ targets) are built in the  `$SOCLABS_PROJECT_DIR/imp/FPGA/targets/` directory.


## ASIC Synthesis
==

To run the ASIC synthesis you will first need to define a `$PHYS_IP` environment variable. This should point to the uncompressed Arm bundles for the TSMC 65nm LP node. 

For a Cadence synthesis flow use:
1. Run `make gen_memories` this will generate the bootrom, and SRAMS using the artisan memory compilers
2. Run `make flist_genus_nanosoc ACCELERATOR=yes ASIC=yes` which will generate the top flist for the genus synthesiser tool
3. Run `make syn_genus ACCELERATOR=yes ASIC=yes` which will run the synthesis

The output from the synthesis will be in the `$SOCLABS_PROJECT_DIR/imp/FPGA/imp/ASIC` directory.
